import { useMemo } from 'react';
import normalizeTokens from './normalizeTokens';
import themeToDict from './themeToDict';

function useThemeDict(theme, language) {
  return useMemo(() => theme && themeToDict(theme, language), [language, theme]);
}

export default function useHighlight({
  theme,
  language,
  code,
  Prism
}) {
  const themeDict = useThemeDict(theme, language);

  const getStyleForToken = ({
    types,
    empty
  }) => {
    const typesSize = types.length;

    if (themeDict === undefined) {
      return undefined;
    }

    if (typesSize === 1 && types[0] === 'plain') {
      return empty ? {
        display: 'inline-block'
      } : undefined;
    }

    if (typesSize === 1 && !empty) {
      return themeDict[types[0]];
    }

    const baseStyle = empty ? {
      display: 'inline-block'
    } : {};
    const typeStyles = types.map(type => themeDict[type]);
    return Object.assign(baseStyle, ...typeStyles);
  };

  const getLineProps = ({
    key,
    className,
    style,
    line: _,
    ...rest
  }) => {
    const output = { ...rest,
      className: 'token-line'
    };

    if (themeDict !== undefined) {
      output.style = themeDict.plain;
    }

    if (style !== undefined) {
      output.style = output.style !== undefined ? { ...output.style,
        ...style
      } : style;
    }

    if (key !== undefined) output.key = key;
    if (className) output.className += ` ${className}`;
    return output;
  };

  const getTokenProps = ({
    key,
    className,
    style,
    token,
    ...rest
  }) => {
    const output = { ...rest,
      className: `token ${token.types.join(' ')}`,
      children: token.content,
      style: getStyleForToken(token)
    };

    if (style !== undefined) {
      output.style = output.style !== undefined ? { ...output.style,
        ...style
      } : style;
    }

    if (key !== undefined) output.key = key;
    if (className) output.className += ` ${className}`;
    return output;
  };

  const tokens = useMemo(() => {
    const grammar = Prism.languages[language];
    const mixedTokens = grammar !== undefined ? Prism.tokenizeWithHooks(code, grammar, language) : [code];
    return normalizeTokens(mixedTokens);
  }, [Prism, code, language]);
  return {
    tokens,
    getLineProps,
    getTokenProps,
    props: {
      className: `prism-code language-${language}`,
      style: themeDict !== undefined ? themeDict.plan : {}
    }
  };
}